<?php
public function exportFiles($template, $theme, $code)
{
	/* Export config file. */
	if(isset($this->config->ui->themes[$template][$theme]))
	{
		$configCode = "<?php\n";
		$configCode .= '$config->ui->themes["' . $template . '"]["' . $code . '"] = ';
		$configCode .= "\n";
		$configCode .= var_export($this->config->ui->themes[$template][$theme], true);
		$configCode .= ";";
		file_put_contents($this->directories->exportConfigPath . "$code.php", $configCode);
	}

	$zfile = $this->app->loadClass('zfile');

	/* Copy customed css file. */
	$customCssFile = $this->directories->exportCssPath . "{$template}_{$code}.css";
	copy($this->getCustomCssFile($template, $theme), $customCssFile);

	/* Copy less file. */
	$lessFile = $this->app->getWwwRoot() . 'theme' . DS . $template . DS . $theme . DS . 'style.less';
	if(file_exists($lessFile)) copy($lessFile, $this->directories->exportLessPath . 'style.less');

    /* Copy upload files. */
    $uploadPath =  $this->app->getDataRoot() . 'upload' . DS . $this->config->site->id;
    if(is_dir($uploadPath)) $zfile->copyDir($uploadPath, $this->directories->exportUploadPath);

	/* Copy source files. */
    $sourcePath = $this->app->getDataRoot() . "source/{$this->config->site->id}/{$template}/{$theme}/";
	if(is_dir($sourcePath)) $zfile->copyDir($sourcePath, $this->directories->exportSourcePath);

	/* Copy slide files. */
	$groups = $this->getUsedSlideGroups($template, $theme);
    $slidePath = $this->app->getDataRoot() . "slides/{$this->config->site->id}/{$theme}/";
	$usedSlides = array();
	foreach($groups as $group) $usedSlides[] = glob($slidePath . DS . "{$group}_*.*");
	foreach($usedSlides as $slides)
	{
		foreach($slides as $slide) copy($slide, str_replace($slidePath . DS, $this->directories->exportSlidePath, $slide));
	}

	/* Upload preview picture. */
	if($_FILES)
	{
		$tmpName  = $_FILES['file']['tmp_name'];
		$fileName = $_FILES['file']['name'];
		move_uploaded_file($tmpName, $this->directories->exportLessPath . 'preview.png');
	}
	else
	{
		$previewImage = $this->app->getWwwRoot() . 'theme' . DS . $template . DS . $theme . DS . 'preview.png';
		copy($previewImage, $this->directories->exportLessPath . 'preview.png');
	}

	/* Export encrypt files. */
	$this->exportEncrypt($template, $theme, $code);

	/* Zip theme files. */
	$this->app->loadClass('pclzip', true);
	$zipFile = dirname($this->directories->exportPath) . DS . $code . '.zip';
	if(file_exists($zipFile)) unlink($zipFile);
	$archive = new PclZip($zipFile);
	$list    = $archive->create($this->directories->exportPath, PCLZIP_OPT_REMOVE_PATH, dirname($this->directories->exportPath));

	if(empty($list)) $this->app->loadClass('zfile')->removeDir(dirname($this->directories->exportPath));
	return $zipFile;
}

public function getCustomCssFile($template, $theme)
{
    $lang = $this->app->getClientLang();
    return $this->app->getDataRoot() . 'css' . DS . $this->config->site->id . DS . "{$template}_{$theme}_{$lang}.css";
}

public function getThemeCssUrl($template, $theme)
{
    $lang = $this->app->getClientLang();
    return $this->config->webRoot . 'data/css/' . $this->config->site->id . "/{$template}_{$theme}_{$lang}.css?v={$this->config->template->customVersion}";
}

public function getThemeInfo($template, $theme)
{
    $this->app->loadClass('Spyc', true);
    $themePath = $this->app->getWwwRoot() . 'theme/' . $template . "/{$theme}/";
    $yamls = glob($themePath . "*.yaml");
    if(empty($yamls)) return false;
    return Spyc::YAMLLoadString(file_get_contents($yamls[0]));
}

/**
 * Get templates.
 * 
 * @access public
 * @return array 
 */
public function getTemplates()
{
    $templates = $this->lang->ui->templates;
    $device    = $this->app->clientDevice;
    foreach($templates as $key => $template)
    {
        if(isset($template['device']) and strpos($template['device'], ",{$device},") === false) unset($templates[$key]);
        $templates[$key]['code'] = $key;
        $templates[$key]['themes'] = $this->lang->ui->themeList->$key;
    }

    return $templates;
}

public function initTheme($device, $template, $theme, $mode)
{
    $themeInited = false;
    $this->app->loadClass('Spyc', true);
    $themePath = $this->app->getWwwRoot() . 'theme/' . $template . "/{$theme}/";
    
    if(isset($this->config->template->installedThemes) and strpos($this->config->template->installedThemes, "{$template}_{$theme}") !== false) $themeInited = true;

    if(!$themeInited)
    {
        if($mode == 'full') $this->loadModel('ui')->clearDB();
        $themeInfo = $this->getThemeInfo($template, $theme);

        $this->app->loadClass('zdb');

        /* Impored sqls. */
        $sqlFile = $mode == 'full' ? 'full.sql' : 'install.sql';
        $sqls = file_get_contents($themePath . $sqlFile);
        $sqls = explode(zdb::LINE_ENDER, $sqls);
 
        foreach($sqls as $sql)
        {
            if(empty($sql)) continue;
            $sql = trim($sql) . ";";
            $this->dbh->query($sql);
        }

        $params = $this->dao->setAutoLang(false)->select('*')->from(TABLE_CONFIG)->where('section')->eq('template')->andWhere('`key`')->eq('custom')->andWhere('lang')->eq('lang')->fetch('value');
        $params = json_decode($params, true);
        $params = zget($params[$template], $themeInfo['theme'], array());

        if(!empty($params)) $this->appendCustom($template, $theme, $params);
        $this->dao->setAutoLang(false)->delete()->from(TABLE_CONFIG)->where('lang')->eq('lang')->exec();
        
        $setting = new stdclass();
        $setting->owner   = 'system';
        $setting->module  = 'common';
        $setting->section = 'template';
        $setting->key     = 'installedThemes';
        $setting->value   = zget($this->config->template, 'installedThemes', '') . ",{$template}_{$theme},";
        $setting->lang    = 'all';
        $this->dao->replace(TABLE_CONFIG)->data($setting)->exec();

        $this->copySource($template, $theme, $mode);
        $this->importSlides($themeInfo, $mode);
    }
    if($mode == 'full') $this->cloneLayout($template, $theme);
    $this->loadModel('setting')->setItems('system.common.site', array('lastUpload' => time()));
    return true;
}

public function importSlides($themeInfo, $mode)
{
    $theme    = $themeInfo['code'];
    $template = $themeInfo['template'];

    if($mode == 'full')
    {
        $slideGroupList = $this->loadModel('tree')->getPairs('', 'slide');

        $slides = array();
        foreach($slideGroupList as $id => $groupName)
        {
            $slides += $this->loadModel('slide')->getList($id);
        }
    }
    else
    {
        $slides = $this->dao->setAutoLang(0)->select('*')->from(TABLE_SLIDE)->where('lang')->eq('lang')->fetchAll();
        if(empty($slides)) return true;

        $group = new stdclass();
        $group->name     = $themeInfo['name'];
        $group->parent   = 0; 
        $group->grade    = 1; 
        $group->type     = 'slide'; 
        $group->postedDate = helper::now(); 

        $this->dao->insert(TABLE_CATEGORY)->data($group)->autoCheck()->exec();
        if(dao::isError()) return false;

        $groupID = $this->dao->lastInsertID();
        $path = ",$groupID,";
        $this->dao->update(TABLE_CATEGORY)->set('path')->eq($path)->where('id')->eq($groupID)->exec();
        $this->dao->setAutoLang(false)->update(TABLE_SLIDE)->set('`group`')->eq($groupID)->set('lang')->eq($this->app->getClientLang())->where('lang')->eq('lang')->exec();

        $slides = $this->loadModel('slide')->getlist($groupID);

    }

    $slidesTmpPath = $this->app->getDataRoot() . "slidestmp/{$theme}/";
    $sourceTmpPath = $this->app->getDataRoot() . "source/{$template}/{$theme}/";

    foreach($slides as $slide)
    {
        foreach($slide as $item => $value)
        {
            if(is_array($value)) $slide->$item = json_encode($value);
        }

        if($slide->backgroundType != 'image') continue;
        $image = $slide->image;
        $basename  = basename($image);
        $extension = pathinfo($image, PATHINFO_EXTENSION);
        if(file_exists($slidesTmpPath . $basename)) 
        {
            $targetPath = $this->app->getDataRoot() . "slides/{$this->config->site->id}/";
            if(!is_dir($targetPath)) mkdir($targetPath, 0777, true);
            copy($slidesTmpPath . $basename, $targetPath . "{$slide->group}_{$slide->id}.{$extension}");
            $slide->image = "/data/slides/{$this->config->site->id}/{$slide->group}_{$slide->id}.{$extension}";
            $this->dao->replace(TABLE_SLIDE)->data($slide)->exec();
        }

        if(file_exists($sourceTmpPath . $basename))
        {
            $slide->image = "/data/source/{$this->config->site->id}/{$template}/{$theme}/{$basename}";
            $this->dao->replace(TABLE_SLIDE)->data($slide)->exec();
        }
    }
    $this->loadModel('setting')->setItems('system.common.site', array('lastUpload' => time()));
    return true;
}

public function setTheme($device, $template, $theme)
{
    $setting = array();
    $setting['name']  = $template;
    $setting['theme'] = $theme;
    return $this->loadModel('setting')->setItems('system.common.template', array($device => json_encode($setting)));
}

public function appendCustom($template, $theme, $params)
{
    $setting = isset($this->config->template->custom) ? json_decode($this->config->template->custom, true) : array();
    $setting[$template][$theme] = $params;

    $config = new stdclass();
    $config->owner   = 'system';
    $config->module  = 'common';
    $config->section = 'template';
    $config->key     = 'custom';
    $config->value   = json_encode($setting);
    $config->lang    = $this->app->getClientLang();
    $this->dao->replace(TABLE_CONFIG)->data($config)->exec();

    return !dao::isError(); 
}

public function copySource($template, $theme, $mode)
{
    $device     = $this->app->clientDevice;;
    $sourcePath = $this->app->getDataRoot() . "source/{$template}/{$theme}/";
    $targetPath = $this->app->getDataRoot() . "source/{$this->config->site->id}/{$template}/{$theme}/";
    $uploadPath = $this->app->getDataRoot() . "upload/{$this->config->site->id}";

    $zfile = $this->app->loadClass('zfile');
    $zfile->copyDir($this->app->getDataRoot() . "themes/{$template}/{$theme}", $uploadPath);
    if(!is_dir($targetPath)) mkdir($targetPath, 0777, true);

    $sourceList = glob($sourcePath . '*');
    if(empty($sourceList)) return true;

    foreach($sourceList as $source) copy($source, $targetPath . basename($source));

    $custom = $this->loadModel('setting')->getItem('owner=system&module=common&section=template&key=custom');

    if($custom)
    {
        $custom = str_replace('\/data\/source\/', "\/data\/source\/{$this->config->site->id}\/", $custom);
        $this->setting->setItems('system.common.template', array('custom' => $custom));
    }

    return true;
}
