<?php
error_reporting(E_ALL);
date_default_timezone_set('Asia/Shanghai');

include 'config.php';
class chanzhi
{
    public $systemRoot  = '';
    public $chanzhiRoot = '';

    /**
     * The construct function.
     * 
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->setConfig();
        $this->setChanzhiRoot();
        $this->getSites();
    }

    /**
     * Set the config.
     * 
     * @access public
     * @return void
     */
    public function setConfig()
    {
        global $config;
        $this->config = $config;
    }

    /**
     * Connect to database.
     * 
     * @access public
     * @return void
     */
    public function connectDB()
    {
        try
        {
            $dbh = new PDO('mysql:host=' . $this->config->db->host, $this->config->db->user, $this->config->db->password);
            $dbh->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_OBJ);
            $dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $dbh->exec("SET NAMES utf8");
            $this->dbh = $dbh;
        }
        catch(PDOException $e)
        {
            echo 'Connection failed: ' . $e->getMessage() . "\n";
            die("connect to db failed.\n");
        }
    }

    /**
     * Set the root path of chanzhi.
     * 
     * @access public
     * @return void
     */
    public function setChanzhiRoot()
    {
        $this->systemRoot  = dirname(dirname(dirname(__FILE__))) . '/';
        $this->chanzhiRoot = dirname($this->systemRoot) . '/';
    }

    /**
     * Get sites on this node.
     * 
     * @access public
     * @return array
     */
    public function getSites()
    {
        $sites = array();
        $this->connectDB();
        $databases = $this->dbh->query("SHOW DATABASES");
        foreach($databases as $row)
        {
            $database = $row->Database;
            if(strpos($database, $this->config->db->prefix) === 0)
            {
                $tables = $this->dbh->query("SHOW TABLES FROM $database");
                if(!$tables->rowCount())
                {
                    /* Drop empty database */
                    $this->dbh->exec("DROP DATABASE `$database`");
                    continue;
                }
                $sites[] = str_replace($this->config->db->prefix, '', $database);
            }
        }
        $this->sites = $sites;
        return $sites;
    }

    /**
     * Get the url of a site.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getSiteURL($subDomain)
    {
        return 'http://' . $subDomain . $this->config->domain . '/';
    }

    /**
     * Get the db name of a site.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getSiteDB($subDomain)
    {
        return $this->config->db->prefix . $subDomain;
    }

    /**
     * Get the attatch directory of a domain.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getAttatchDir($subDomain)
    {
        return $this->chanzhiRoot . "www/data/upload/" . $subDomain{0} . "/$subDomain/";
    }

    /**
     * Init the backup directory of a domain.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function initBackDir($subDomain)
    {
        $dir = $this->chanzhiRoot . 'backup/' . $subDomain;
        if(!is_dir($dir)) mkdir($dir, 0755, true);
        return $dir;
    }

    /**
     * Exec a commad with client shell.
     * 
     * @param  string    $commad 
     * @access public
     * @return void
     */
    public function cliExec($commad)
    {
        $commad = "cd {$this->systemRoot}bin/;./xrcli {$commad}";
        echo `$commad`;
    }

    public function sync2CDN()
    {
        if(is_file($this->systemRoot . '/tmp/log/sync2CDN'))
        {
            if((time() - filemtime($this->systemRoot . '/tmp/log/sync2CDN')) / 60 >= 10)
            {
                echo date('Y-m-d H:i:s') . " Stop process to force!\n";
                $pids = `ps aux|grep sync2cdn.php |awk '$0!~/grep sync2cdn.php/{print $2}'`;
                if($pids)
                {
                    $pids = explode("\n", trim($pids));
                    array_pop($pids);
                    foreach($pids as $pid) `kill $pid`;
                }
                unlink($this->systemRoot . '/tmp/log/sync2CDN');
            }
            else
            {
                die(date('Y-m-d H:i:s') . " sync2CDN is running!\n");
            }
        }
        touch($this->systemRoot . '/tmp/log/sync2CDN');

        $dataPath = 'data/';
        $wwwRoot  = $this->chanzhiRoot . 'www/';
        exec("rsync -avP -e'ssh -p {$this->config->cdn->port}' --delete $wwwRoot$dataPath {$this->config->cdn->account}@{$this->config->cdn->host}:{$this->config->cdn->path}$dataPath", $output, $return);
        $syncFiles = array();

        foreach($output as $key => $value)
        {
            if(strpos($value, '100%') !== false)
            {
                $syncFile = $output[$key -1];
                $firstDir = substr($syncFile, 0, strpos($syncFile, '/'));
                if($firstDir != 'upload' and $firstDir != 'source' and $firstDir != 'slides') continue;

                $level = count(explode('/', $syncFile));
                if($level == 6) //source
                {
                    list($source, $letter, $domain, $template, $theme, $fileName) = explode('/', $syncFile);
                    $syncFiles[$domain][] = $source . '/' . $letter . '/' . $domain . '/' . $template . '/' . $theme . '/' . $fileName;
                }
                else if($level == 4) //slides
                {
                    list($slide, $letter, $domain, $fileName) = explode('/', $syncFile);
                    $syncFiles[$domain][] = $slide . '/' . $letter . '/' . $domain . '/' . $fileName;
                }
                else //upload
                {
                    list($upload, $letter, $domain, $path, $fileName) = explode('/', $syncFile);
                    $syncFiles[$domain][] = $path . '/' . $fileName;
                }
            }
        }

        foreach($syncFiles as $site => $files)
        {
            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");

            foreach($files as $file)
            {
                echo date('Y-m-d H:i:s') . " Sync $file.\n";
                try{ $this->dbh->exec("update eps_file set cdn = '{$this->config->cdn->host}' where pathname = '$file'"); }catch(PDOException $e){echo $e->getMessage() . "\n";}
            }
            echo date('Y-m-d H:i:s') . " Sync file for $site";
            echo " finished \n";
        }
        unlink($this->systemRoot . '/tmp/log/sync2CDN');
    }

    /**
     * Backup database and files.
     * 
     * @access public
     * @return void
     */
    public function backup()
    {
        $sites = $this->getSites();
        foreach($sites as $site)
        {
            $backDir = $this->initBackDir($site);
            $this->execBackup($site, $backDir);
            echo "--- $site end ---\n";
        }

        $backRoot = $this->chanzhiRoot . 'backup/';
        echo `sudo chmod -R 777 $backRoot`;
    }

    public function connect($host, $port, $user, $pubKey, $privKey)
    {
        $this->connect = ssh2_connect($host, $port);
        if(!ssh2_auth_pubkey_file($this->connect, $user, $pubKey, $privKey))
        {
            echo "$host connect fail!\n";
            return 'fail';
        }

        return 'ok';
    }

    public function sshExec($cmd)
    {
        $result = array();

        if(!$this->connect)
        {
            $result['log']   = '';
            $result['error'] = "Can not connect $this->domain!";
            return $result;
        }

        $result['log']   = '';
        $result['error'] = '';

        $stream      = ssh2_exec($this->connect, $cmd);
        $errorStream = ssh2_fetch_stream($stream, SSH2_STREAM_STDERR);

        stream_set_blocking($errorStream, true);
        stream_set_blocking($stream, true);

        $result['log']   = stream_get_contents($stream);
        $result['error'] = stream_get_contents($errorStream);

        // Close the streams        
        fclose($stream);
        fclose($errorStream);

        return $result;
    }

    /**
     * Upgrade to latest version.
     * 
     * @access public
     * @return void
     */
    public function upgrade($fromVersion, $toVersion)
    {
        $sites = $this->getSites();
        $sqls  = null;
        //$sqls  = $this->getUpgradeSQL($fromVersion);
        foreach($sites as $site)
        {
            echo date('Y-m-d H:i:s') . " upgradeing $site\n";
            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");

            if(!empty($sqls))
            {
                foreach($sqls as $sql) try{ $this->dbh->exec($sql); }catch(PDOException $e){echo $e->getMessage() . "\n";}
            }

            $siteURL = $this->getSiteURL($site);
            $this->cliExec("{$siteURL}api-getModel-upgrade-fixHeaderBlock.html;");

            $this->updateVersion($toVersion);
            echo " finished \n";
        }
    }

    /**
     * Get the sqls to execute.
     * 
     * @param  string    $fromVersion 
     * @access public
     * @return array
     */
    public function getUpgradeSQL($fromVersion)
    {
        $fromFile = '../../db/upgrade' . $fromVersion . '.sql';
        if(!file_exists($fromFile)) return array();
        $sqls = file_get_contents($fromFile);
        $sqls = explode(';', $sqls);
        foreach($sqls as $key => $sql)
        {
            if(!trim($sql)) unset($sqls[$key]);
        }
        return $sqls;
    }

    /**
     * Get the ext sqls to execute.
     * 
     * @param  string    $fromVersion 
     * @access public
     * @return array
     */
    public function getExtUpgradeSQL($fromVersion)
    {
        $fromFile = '../../../db/upgrade' . $fromVersion . '.sql';
        if(!file_exists($fromFile)) return array();
        $sqls = file_get_contents($fromFile);
        $sqls = explode(';', $sqls);
        foreach($sqls as $key => $sql)
        {
            if(!trim($sql)) unset($sqls[$key]);
        }
        return $sqls;
    }

    /**
     * Update the version.
     * 
     * @access public
     * @return void
     */
    public function updateVersion($version)
    {
        try{ $this->dbh->exec("UPDATE eps_config SET value='$version' WHERE `key` = 'version'"); }catch(PDOException $e){echo $e->getMessage() . "\n";}
    }

    /**
     * Update SQL of all site.
     * 
     * @access public
     * @return void
     */
    public function updateSQL()
    {
        $sites = $this->getSites();
        $fromFile = './update.sql';
        $sqls = file_get_contents($fromFile);
        $sqls = explode(';', $sqls);
        foreach($sqls as $key => $sql)
        {
            if(!trim($sql)) unset($sqls[$key]);
        }
        foreach($sites as $site)
        {
            echo date('Y-m-d H:i:s') . " upgrading $site";
            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");

            foreach($sqls as $sql) try{ $this->dbh->exec($sql); }catch(PDOException $e){echo $e->getMessage() . "\n";}
            echo " finished \n";
        }
    }

    /**
     * Delete sites 
     * 
     * @access public
     * @return void
     */
    public function deleteSites()
    {
        $sites = $this->getSites();

        $count = 0;
        $deleteBack = '/home/data/chanzhidelete/';
        foreach($sites as $site)
        {
            $deleted = $this->checkDelete($site);
            $result  = $deleted ? "$site 删除\n" : "$site 保留 \n";
                
            if(!$deleted) continue;

            echo date('Y-m-d H:i:s') . " Deleting $site\n";
            /* Backup site*/
            $backDir = $deleteBack . $site . '/' . date('Ymd') . '/';
            if(!is_dir($backDir)) `sudo mkdir -m 777 -p $backDir`;
            $this->execBackup($site, $backDir);

            /* Delete dabase. */
            $siteDB = $this->getSiteDB($site);
            $user   = 'u_' . $site;
            $sqls   = array();
            $sqls[] = "DROP DATABASE IF EXISTS $siteDB";
            $sqls[] = "DELETE FROM mysql.user WHERE user='$user' AND host='localhost'";
            $sqls[] = "DELETE FROM mysql.db WHERE user='$user' AND host='localhost'";
            $sqls[] = 'FLUSH PRIVILEGES';
            foreach($sqls as $sql) $this->dbh->exec($sql);

            /* Delete attatch. */
            $attatchDir = $this->getAttatchDir($site);
            if(is_dir($attatchDir)) `sudo rm -fr $attatchDir`;

            /* Delete backup. */
            $backDir = $this->initBackDir($site);
            if(is_dir($backDir)) `sudo rm -fr $backDir`;

            echo "---delete $site finish ---\n";
        }
    }

    /**
     * Exec backup 
     * 
     * @param  string    $site 
     * @param  string    $backDir 
     * @access public
     * @return void
     */
    public function execBackup($site, $backDir)
    {
        /* Backup database. */
        $siteDB    = $this->getSiteDB($site);
        $dbFile    = "$backDir/$site.sql";
        $backupKey = $this->getBackupKey($site);
        $backCMD   = "mysqldump $siteDB -u {$this->config->db->user} -p{$this->config->db->password} > $dbFile\n";
        if($backupKey == '') 
        {
            $backCMD .= "cd $backDir; zip -r $backDir/$site.sql.zip *.sql\n";
        }
        else
        {
            $backCMD .= "cd $backDir; zip -r -P $backupKey $backDir/$site.sql.zip *.sql\n";
        }
        $backCMD .= "rm -fr $dbFile\n";

        /* Backup file. */
        $attatchDir = $this->getAttatchDir($site);
        if(is_dir($attatchDir))
        {
            if($backupKey == '') 
            {
                $backCMD .= "cd $attatchDir; sudo zip -r $backDir/$site.file.zip *\n";
            }
            else
            {
                $backCMD .= "cd $attatchDir; sudo zip -r -P $backupKey $backDir/$site.file.zip *\n";
            }
        }

        /* delete old file. */
        if(file_exists("$backDir/$site.sql.bz2")) $backCMD .= "rm $backDir/$site.sql.bz2\n";
        if(file_exists("$backDir/$site.file.bz2")) $backCMD .= "rm $backDir/$site.file.bz2\n";

        echo date('Y-m-d H:i:s') . " backuping $site\n";
        `$backCMD`;
    }

    public function backupDB($site, $backDir, $fileName = '')
    {
        /* Backup database. */
        $siteDB    = $this->getSiteDB($site);
        $today     = date('ymd'); 
        $dbFile    = "$backDir/$site.sql";
        $backupKey = $this->getBackupKey($site);
        $backCMD   = "mysqldump $siteDB -u {$this->config->db->user} -p{$this->config->db->password} > $dbFile\n";
        $backCMD   .= "mkdir /home/z/backup/{$site}\n";
        if($backupKey == '') 
        {
            $backCMD .= "cd $backDir; zip -r /home/z/backup/{$site}/{$fileName}.zip *.sql\n";
        }
        else
        {
            $backCMD .= "cd $backDir; zip -r -P $backupKey /home/z/backup/{$site}/{$fileName}.zip *.sql\n";
        }
        $backCMD .= "rm -fr $dbFile\n";

        echo date('Y-m-d H:i:s') . " backuping $site sql\n";
        `$backCMD`;
    }

    public function backupAll()
    {
        $month       = date('Ym/');
        $today       = date('Ymd');
        $dbBack      = '/home/back/db/' . $month;
        $confBack    = '/home/back/conf/' . $month;
        $chanzhiBack = '/home/back/z/' . $month;

        if(!is_dir('/home/back/'))
        {
            `sudo mkdir /home/back/`;
            `sudo chown -R z:z /home/back/`;
        }
        if(!is_dir($dbBack))      `mkdir -p $dbBack`;
        if(!is_dir($confBack))    `mkdir -p $confBack`;
        if(!is_dir($chanzhiBack)) `mkdir -p $chanzhiBack`;

        `touch  {$dbBack}/tmp.txt`;
        `touch  {$confBack}/tmp.txt`;
        `touch  {$chanzhiBack}/tmp.txt`;

        /* 4. Remove old backups. */
        echo date('Y-m-d H:i:s') . " remove old files now...\n";
        `find /home/back/db -mtime +7 |xargs rm`;
        `find /home/back/z -mtime +3 |xargs rm`;
        `find /home/back/conf -mtime +7 |xargs rm`;
        `rmdir /home/back/conf/* ; rmdir /home/back/z/* ; rmdir /home/back/db/*;`;

        `rm {$dbBack}/tmp.txt`;
        `rm {$confBack}/tmp.txt`;
        `rm {$chanzhiBack}/tmp.txt`;

        /* Backup databases. */
        echo date('Y-m-d H:i:s') . " backuping databases now...\n";

        $this->connectDB();
        $databases = $this->dbh->query("SHOW DATABASES");
        $tmpDir    = $dbBack . $today;
        if(!is_dir($tmpDir)) `mkdir $tmpDir`;
        foreach($databases as $row)
        {
            $dbName = $row->Database;
            if($dbName == 'information_schema' or $dbName == 'mysql' or $dbName == 'phpmyadmin') continue;

            $dbFile = "$tmpDir/$dbName.sql";
            $dbCMD  = "mysqldump $dbName  -u {$this->config->db->user} -p{$this->config->db->password} --skip-lock-tables > $dbFile\n";
            echo `$dbCMD`;
        }
        $dbCMD  = "cd $dbBack && tar cjvf $dbBack$today.tar.bz2 $today\n";
        $dbCMD .= "rm -fr $tmpDir\n";
        echo `$dbCMD`;
        echo date('Y-m-d H:i:s') . " backuping databases done.\n\n";

        /* 2. Backup home directory of chanzhi account. */
        $tmpDir = $chanzhiBack . $today;
        if(!is_dir($tmpDir)) `mkdir $tmpDir`;
        $chanzhiBackFile = "{$chanzhiBack}$today.tar.bz2";
        $chanzhiCMD  = "rsync -av --exclude='tmp' --exclude='backup' --exclude='todelete' /home/z/ $tmpDir\n";
        $chanzhiCMD .= "cd $tmpDir && rm -rf ext \n";
        $chanzhiCMD .= "cd $chanzhiBack && tar cjvf $chanzhiBackFile $today\n";
        $chanzhiCMD .= "rm -rf $tmpDir\n";

        echo date('Y-m-d H:i:s') . " backuping chanzhi directory now...\n";
        echo `$chanzhiCMD`;
        echo date('Y-m-d H:i:s') . " backuping chanzhi directory done.\n\n";

        /* 3. Backup config files. */
        $tmpDir = $confBack . $today;
        if(!is_dir($tmpDir)) `mkdir $tmpDir`;
        echo date('Y-m-d H:i:s') . " backuping config files now...\n";
        echo `sudo cp -fr /etc/nginx/ $tmpDir`;
        echo `sudo cp -fr /etc/rc.local $tmpDir`;
        echo `sudo cp -fr /etc/ssh/ $tmpDir`;
        echo `crontab -l > crontab; mv crontab $tmpDir`;
        echo `sudo chown -R z $tmpDir`;
        echo `cd $confBack && tar cjvf $confBack/$today.tar.bz2 $today`;
        echo `rm -fr $tmpDir`;
        echo date('Y-m-d H:i:s') . " backuping config files done.\n\n";

        /* 4. Remove old backups. */
        echo date('Y-m-d H:i:s') . " remove old files now...\n";
        `find /home/back/db -mtime +7 |xargs rm`;
        `find /home/back/z -mtime +3 |xargs rm`;
        `find /home/back/conf -mtime +7 |xargs rm`;
        `rmdir /home/back/conf/* ; rmdir /home/back/z/* ; rmdir /home/back/db/*;`;
        echo date('Y-m-d H:i:s') . " remove old files done.\n\n";

        echo date('Y-m-d H:i:s') . " backuping done!\n";
    }

    public function getBackupKey($site)
    {
        $backupInfo = $this->getBackupInfo($site);
        return isset($backupInfo['password']) ? $backupInfo['password'] : '';
    }

    public function getBackupInfo($site)
    {
        if(!class_exists('zcrypt')) include '/home/z/node/system/lib/zcrypt/zcrypt.class.php';
        $dbName = $this->getSiteDB($site);
        $this->dbh->exec("USE $dbName");

        $results = $this->dbh->query("select `key`,`value` from eps_config where `key` in ('backupDate', 'backupKey', 'backupAccount') and `module` = 'common' and `section` = 'global' and `owner` = 'system' and lang='all'")->fetchAll(PDO::FETCH_ASSOC);
        $backupInfo = array();
        if(!empty($results))
        {
            foreach($results as $info)
            {
                if($info['key'] == 'backupKey') $backupKey = $info['value'];
                if(isset($backupKey)) $backupInfo['password'] = zcrypt::decrypt(base64_decode($backupKey), $this->config->cryptKey);

                if($info['key'] == 'backupDate')    $backupInfo['backupDate'] = $info['value'];
                if($info['key'] == 'backupAccount') $backupInfo['account']    = $info['value'];
            }
            if(!empty($backupInfo['account']) and !empty($backupInfo['account']) and $backupInfo['backupDate'] == date('Y-m-d')) return $backupInfo;
        }
        return array();
    }

    /**
     * Count yestoday Traffic 
     * 
     * @access public
     * @return void
     */
    public function countYestodayTraffic()
    {
        global $config;
        include_once("{$this->systemRoot}framework/helper.class.php");
        include_once("{$this->systemRoot}config/domain.php");
        $this->config = $config;
        $config->domainPostfix .= "|n1.chanzhi.net|xxx|zsite.net|zsite.com|";
        $yestoday = date('Y-m-d', strtotime('last day'));

        /* Get cdn traffic. */
        /*
        $resultFile = RESULT_FILE_PATH;
        exec("rsync -avP -e'ssh -p {$this->config->cdn->port}' {$this->config->cdn->account}@{$this->config->cdn->host}:$resultFile $resultFile", $output);
        print_r($output);
        $result = json_decode(file_get_contents($resultFile));
        if(!isset($result->$yestoday)) exit('not get CDN data.');
        $cdnTraffic = get_object_vars($result->$yestoday);
         */

        $logFile = LOG_FILE_PATH;
        $fh      = @fopen($logFile, 'r');
        if(!$fh) exit('read log file permission denied.');
        while(!feof($fh))
        {
            $line = fgets($fh);
            if($line == '') continue;
            $re = preg_match_all("/\[(.*?)\] ([a-zA-Z0-9.-]+?) (\d+) ([0-9.]+?) (\d+) \[(.+?)\] \[(.+?)\] \[(.+?)\]/", $line, $out);
            if(!$re) continue;

            $info = new stdclass();
            $info->host = trim($out[2][0]);
            if(strpos($info->host, '.') === false) continue;
            $info->siteCode = helper::getSiteCode($info->host);

            $info->size = (int)$out[3][0];
            $info->ip   = $out[4][0];
            $info->agent = trim($out[8][0]);

            if(!isset($masterTraffic[$info->siteCode])) $masterTraffic[$info->siteCode] = 0;
            $masterTraffic[$info->siteCode] += $info->size;

            if(!isset($ip[$info->siteCode])) $ip[$info->siteCode] = array();
            $ip[$info->siteCode][$info->ip] = $info->ip;

            if(!isset($pv[$info->siteCode])) $pv[$info->siteCode] = 0;
            $pv[$info->siteCode] ++;

            if(!isset($totalTraffic[$info->siteCode])) $totalTraffic[$info->siteCode] = 0;
            $totalTraffic[$info->siteCode] += $info->size;
        }

        fclose($fh);
        //asort($cdnTraffic);
        asort($masterTraffic);
        asort($totalTraffic);

        $traffic = new stdclass();
        $traffic->cdn    = 0;
        $traffic->master = $masterTraffic;
        $traffic->total  = $totalTraffic;
        $traffic->ip     = array();
        $traffic->pv     = $pv;

        foreach($ip as $siteCode => $ipAddress)
        {
            $traffic->ip[$siteCode] = count($ipAddress);
        }

        //数据库大小
        $sql    = 'SELECT sum(DATA_LENGTH)+sum(INDEX_LENGTH) AS size FROM information_schema.TABLES;';
        $result = $this->dbh->query($sql)->fetch(); 
        $traffic->db = $result->size / 1024;

        //附件大小
        $uploadPath = $this->chanzhiRoot . 'www/data/upload/';
        if(!is_dir($uploadPath));
        exec("du -s $uploadPath", $output, $return);
        preg_match_all("/(\d+).*/", $output[0], $out);
        $traffic->file = $out[1][0];

        // 目前直接生成文件，如果有节点不在同一个服务器上，需要修改逻辑
        file_put_contents('/tmp/trafficYestoday', json_encode($traffic));

        $sites = $this->getSites();
        foreach($sites as $site)
        {
            echo date('Y-m-d H:i:s') . " Counting $site\n";
            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");

            $cdn    = 0;
            $master = isset($masterTraffic[$site]) ? round($masterTraffic[$site] / 1024, 2) : 0;
            $total  = isset($totalTraffic[$site]) ? round($totalTraffic[$site] / 1024, 2) : 0;
            $sql = "REPLACE INTO eps_traffic (`total`, `cdn`, `master`, `date`) VALUES ($total, $cdn, $master, '$yestoday')";
            try { $this->dbh->exec($sql); } catch (PDOException $e) { echo $e->getMessage() . "\n"; }

            echo " finished \n";
        }
    }

    /**
     * Compute usage and traffic.
     * 
     * @access public
     * @return void
         */
        public function computeUsageAndTraffic()
        {
            $sites = $this->getSites();
            foreach($sites as $site)
            {
                $siteDB = $this->getSiteDB($site);
                $this->dbh->exec("USE $siteDB");

                $db      = $this->getDbUsage();
                $file    = $this->getFileUsage($site);
                $usage   = $db + $file;
                $traffic = $this->getTraffic();

                $sqls = array();
                $sqls[] = "REPLACE INTO eps_config (`owner`, `module`, `section`, `key`, `value`, `lang`) VALUES ('system', 'common', 'global', 'usage', $usage, 'all')";
                $sqls[] = "REPLACE INTO eps_config (`owner`, `module`, `section`, `key`, `value`, `lang`) VALUES ('system', 'common', 'global', 'traffic', $traffic->total, 'all')";
                $sqls[] = "REPLACE INTO eps_config (`owner`, `module`, `section`, `key`, `value`, `lang`) VALUES ('system', 'common', 'global', 'cdnTraffic', $traffic->cdn, 'all')";
                $sqls[] = "REPLACE INTO eps_config (`owner`, `module`, `section`, `key`, `value`, `lang`) VALUES ('system', 'common', 'global', 'masterTraffic', $traffic->master, 'all')";

                foreach($sqls as $sql) try{ $this->dbh->exec($sql); } catch(PDOException $e) { echo $e->getMessage() . "\n"; }
            }
        }

        /**
         * get the usage of db
         *
         * @access public
         * @return float
         */
        public function getDbUsage()
        {
            $stmt    = $this->dbh->query("SHOW TABLE STATUS");
            $tables  = $stmt->fetchALL();
            $dbUsage = 0;
            foreach($tables as $table)
            {
                $dbUsage += $table->Data_length + $table->Index_length;
            }
            $dbUsage /= 1024;
            return round($dbUsage, 2);
        }

        /**
         * get size of file
         *
         * @access public
         * @return float
         */
        public function getFileUsage($site)
        {
            $uploadPath = $this->chanzhiRoot . 'www/data/upload/' . substr($site, 0, 1) . "/$site";
            if(!is_dir($uploadPath)) return 0;
            exec("du -s $uploadPath", $output, $return);
            preg_match_all("/(\d+).*/", $output[0], $out);
            return $out[1][0];
        }

        /**
         * getTraffic 
         * 
         * @access public
         * @return float
         */
        public function getTraffic()
        {
            $totalTraffic  = 0;
            $cdnTraffic    = 0;
            $masterTraffic = 0;

            $firstDay = date('Y-m') . '-01';
            $traffic  = $this->dbh->query("SELECT SUM(cdn) AS cdn, SUM(master) AS master, SUM(total) AS total FROM eps_traffic where `date` >= '$firstDay'")->fetch();
            return $traffic;
        }

        /**
         * Create customer css.
         * 
         * @access public
         * @return void
         */
        public function createCustomerCss()
        {
            $sites = $this->getSites();
            foreach($sites as $site)
            {
                echo date('Y-m-d H:i:s') . " create css for $site\n";

                $siteDB = $this->getSiteDB($site);
                $this->dbh->exec("USE $siteDB");

                $siteURL = $this->getSiteURL($site);
                $this->cliExec("{$siteURL}api-createcustomcss.html;");
                echo " finished \n";
            }
        }

        /**
         * Get site info from plat db.
         * 
         * @param  string    $site 
         * @access public
         * @return object
         */
        public function getSiteFromPlat($site)
        {
            return $this->dbh->query("select * from chanzhi.eps_site where code = '$site'")->fetch();
        }

        /**
         * Mark a site deleted in plat db.
         * 
         * @param  sting    $site 
         * @access public
         * @return void
         */
        public function deleteFromPlat($site)
        {
            $this->dbh->exec("update chanzhi.eps_site set status = 'deleted' where code = '$site'");
        }

        /**
         * Check whether a site should be deleted.
         * 
         * @param  string    $site 
         * @access public
         * @return bool
         */
        public function checkDelete($site)
        {
            if($site == 'site') return false;
            $siteInfo = $this->getSiteFromPlat($site);
            if(empty($siteInfo)) return true;

            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");
            $service = $this->dbh->query("select `value` from eps_config where `key` = 'endDate' and `module` = 'common' and `section` = 'service' and `owner` = 'system'")->fetch();

            if($service and strtotime($service->value) < strtotime('-3 month')) return true;

            if($service and strtotime($service->value) < strtotime('-1 month'))
            {
                $this->dbh->exec("REPLACE INTO eps_config set value='1',`key` = 'deleted', `module` = 'common', `section` = 'global', `owner` = 'system', lang='all'");
                $this->deleteFromPlat($site);
                return false;
            }
            return false;
        }
}
