<?php
/**
 * Create the database for a site.
 * 
 * @param  string    $siteID
 * @access public
 * @return void
 */
public function createDB($siteID)
{
    $database = $this->getNameOfDB($siteID);
    $sqls[] = "DROP DATABASE IF EXISTS $database";
    $sqls[] = "CREATE DATABASE $database;";
    foreach($sqls as $sql) $this->dbh->query($sql);
    return !dao::isError();
}

/**
 * Create tables for a site.
 * 
 * @param  string    $siteID 
 * @access public
 * @return void
 */
public function createTable($siteID, $type = 'basic')
{
    $dbName  = $this->getNameOfDB($siteID);
    $sysRoot = $type == 'basic' ? $this->app->getAppRoot() : dirname($this->app->getAppRoot()) . '/prosystem/';
    $tables  = explode(';', file_get_contents($sysRoot . 'db/chanzhi.sql'));
    foreach($tables as $table)
    {
        $table = trim($table);
        if(empty($table)) continue;

        if(strpos($table, 'DROP') !== false and $this->post->clearDB != false)
        {
            $table = str_replace('--', '', $table);
        }
        $table = str_replace('eps_', $this->config->db->prefix, $table);
        $table = str_replace('`' . $this->config->db->prefix, $dbName . '.`' . $this->config->db->prefix, $table);

        if(!$this->dbh->query($table)) return false;
    }
    return !dao::isError();
}

/**
 * Execute saasSQL 
 * 
 * @param  string    $siteID 
 * @access public
 * @return bool
 */
public function executeSaasSQL($siteID)
{
    $dbName = $this->getNameOfDB($siteID);
    $sqlFile = dirname($this->app->getAppRoot()) . '/db/install.sql';
    if(file_exists($sqlFile))
    {   
        $sqls = explode(';', file_get_contents($sqlFile));
        foreach($sqls as $sql)
        {   
            $sql = trim($sql);
            if(empty($sql)) continue;

            $sql = str_replace('`eps_', $dbName . '.`eps_', $sql);

            if(!$this->dbh->query($sql)) return false;
        }   
    }   
    return true;
}

/**
 * Set admin for a site.
 * 
 * @param  string    $siteID 
 * @param  string    $adminAccount 
 * @access public
 * @return void
 */
public function setAdmin($siteID, $adminAccount, $adminPassword, $adminEmail)
{
    /* Set tables. */
    $database  = $this->getNameOfDB($siteID);
    $userTable = $database . '.' . $this->config->db->prefix . 'user';

    $admin = new stdclass();
    $admin->account  = $adminAccount;
    $admin->realname = $adminAccount;
    $admin->admin    = 'super';
    $admin->join     = date('Y-m-d H:i:s');
    $admin->password = $adminPassword;
    $admin->email    = $adminEmail;
    $admin->lang     = 'all';
    $this->dao->insert($userTable)->data($admin)->exec();
}

/**
 * Sync password and email for admin.
 * 
 * @param  string    $siteID 
 * @param  string    $adminAccount 
 * @param  string    $adminPassword 
 * @param  string    $adminEmail 
 * @access public
 * @return void
 */
public function updateAdmin($siteID, $adminAccount, $adminPassword, $adminEmail)
{
    $database  = $this->getNameOfDB($siteID);
    $userTable = $database . '.' . $this->config->db->prefix . 'user';
    $admin = $this->dao->select('*')->from($userTable)->where('account')->eq($adminAccount)->fetch();

    $data = new stdclass();
    $data->password = !empty($adminPassword) ? $adminPassword : $admin->password;
    $data->email    = !empty($adminEmail) ? $adminEmail : $admin->email;

    $this->dao->update($userTable)->data($data)->where('account')->eq($adminAccount)->exec();
}

/**
 * Set config.
 * 
 * @param  string $siteID 
 * @param  string $private 
 * @param  string $status 
 * @access public
 * @return void
 */
public function setConfig($siteID, $private, $status, $plan, $endDate)
{
    /* Set tables. */
    $database    = $this->getNameOfDB($siteID);
    $configTable = $database . '.' . $this->config->db->prefix . 'config';

    /* The serial number. */
    $item = new stdclass();
    $item->owner   = 'system';
    $item->module  = 'common';
    $item->section = 'global';
    $item->key     = 'version';
    $item->value   = $this->config->version; 
    $item->lang    = 'all';
    $this->dao->replace($configTable)->data($item)->exec();

    $item->key   = 'siteStatus';
    $item->value = $status;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->key   = 'openedDate';
    $item->value = date('Y-m-d H:i:s');
    $this->dao->replace($configTable)->data($item)->exec();

    $item->key   = 'agreed';
    $item->value = 1;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->key   = 'private';
    $item->value = $private;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->section = 'service';
    $item->key   = 'plan';
    $item->value = $plan;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->section = 'service';
    $item->key     = 'endDate';
    $item->value   = $endDate;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->section = 'site';
    $item->key     = 'lang';
    $item->value   = 'zh-cn';
    $this->dao->replace($configTable)->data($item)->exec();

    $item->section = 'site';
    $item->key     = 'modules';
    $item->value   = 'user,product,shop,article';
    $this->dao->replace($configTable)->data($item)->exec();
}

/**
 * Update service of endDate.
 * 
 * @param  string $siteID 
 * @param  string $endDate 
 * @access public
 * @return void
 */
public function updateService($siteID, $status, $plan, $endDate)
{
    $database    = $this->getNameOfDB($siteID);
    $configTable = $database . '.' . $this->config->db->prefix . 'config';

    $item = new stdclass();
    $item->owner   = 'system';
    $item->module  = 'common';
    $item->section = 'service';
    $item->key     = 'endDate';
    $item->value   = $endDate;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->key   = 'plan';
    $item->value = $plan;
    $this->dao->replace($configTable)->data($item)->exec();

    $item->section = 'global';
    $item->key     = 'siteStatus';
    $item->value   = $status;
    $this->dao->replace($configTable)->data($item)->exec();
}

/**
 * get the database name for a site.
 * 
 * @param  string    $siteID 
 * @access public
 * @return string
 */
public function getNameOfDB($siteID)
{
    return '`' . $this->config->codePrefix . $siteID . '`';
}

/**
 * Change database for a domain.
 * 
 * @param  string    $domain 
 * @param  string    $oldDomain 
 * @access public
 * @return void
 */
public function changeDB($domain, $oldDomain)
{
    $oldDatabase = $this->getNameOfDB($oldDomain);
    $database    = $this->getNameOfDB($domain);

    $this->dbh->exec('USE ' . $oldDatabase);
    $tables = $this->dbh->query('show tables')->fetchAll(PDO::FETCH_COLUMN);

    $sqls[] = "DROP DATABASE IF EXISTS $database";
    $sqls[] = "CREATE DATABASE $database;";
    foreach($tables as $table)
    {
        $sqls[] = "DROP TABLE IF EXISTS $database.$table;";
        $sqls[] = "CREATE TABLE $database.$table LIKE $table;";
        $sqls[] = "INSERT INTO $database.$table SELECT * FROM $table;";
    }

    foreach($sqls as $sql) $this->dbh->query($sql);
}

/**
 * Delete old database and user after change site.
 * 
 * @param  string    $siteID
 * @access public
 * @return void
 */
public function deleteDB($siteID)
{
    $database = $this->getNameOfDB($siteID);

    $sqls[] = "DROP DATABASE IF EXISTS $database;";
    $sqls[] = 'FLUSH PRIVILEGES';
    foreach($sqls as $sql) $this->dbh->query($sql);
}

/**
 * Move files to new path.
 * 
 * @param  string    $domain 
 * @param  string    $oldDomain 
 * @access public
 * @return string
 */
public function moveFiles($domain, $oldDomain)
{
    $domain    = strpos($domain, 'www.') === 0 ? substr($domain, 4) : $domain;
    $oldDomain = strpos($oldDomain, 'www.') === 0 ? substr($oldDomain, 4) : $oldDomain;

    $movePathList = array('upload', 'source', 'slides', 'css');
    foreach($movePathList as $movePath)
    {
        $oldSavePath = $this->app->getWwwRoot() . "sites/" . $oldDomain . DS . $movePath;
        if(file_exists($oldSavePath))
        {
            $savePath = $this->app->getWwwRoot() . "sites/" . $domain . DS . $movePath;
            @mkdir($savePath, 0777, true);
            @rename($oldSavePath, $savePath);
        }
    }
    return true;
}

/**
 * Fix path for file in sql.
 * 
 * @param  string    $domain 
 * @param  string    $oldDomain 
 * @access public
 * @return string
 */
public function fixFilePath($domain, $oldDomain)
{
    $database = $this->getNameOfDB($domain);
    $this->dbh->exec('USE ' . $database);

    $this->dao->update(TABLE_CONFIG)->set('`value`')->eq(time())->where('`key`')->eq('lastUpload')->exec();

    $tables = $this->dbh->query('show tables')->fetchAll(PDO::FETCH_COLUMN );
    foreach($tables as $table)
    {
        $fields = $this->dbh->query("DESC {$table}")->fetchAll(PDO::FETCH_COLUMN );
        $rows = $this->dao->select('*')->from($table)->fetchAll();
        foreach($rows as $row)
        {
            foreach($fields as $field)
            {
                $oldSavePath = "sites/{$oldDomain}/data/upload/";
                $savePath    = "sites/{$domain}/data/upload/";

                if(strpos($row->$field, $oldSavePath) !== false)
                {
                    $value = str_replace($oldSavePath, $savePath, $row->$field);
                    $this->dao->update($table)->set("`{$field}`")->eq($value)->where("`{$field}`")->eq($row->$field)->exec();
                }
                else
                {
                    $oldSavePath = "sites\/" . $oldDomain . "\/data\/upload";
                    $savePath    = "sites\/" . $domain . "\/data\/upload";
                    if(strpos($row->$field, $oldSavePath) !== false)
                    {
                        $value = str_replace($oldSavePath, $savePath, $row->$field);
                        $this->dao->update($table)->set("`{$field}`")->eq($value)->where("`{$field}`")->eq($row->$field)->exec();
                    }
                }
            }
        }
    }
    return !dao::isError();
}

/**
 * Backup site.
 * 
 * @param  string    $siteID 
 * @access public
 * @return bool
 */
public function backupSite($siteID)
{
    $id2Domain = $this->config->site->id2Domain;
    $domain    = $id2Domain[$siteID];

    $backupPath = $this->app->getTmpRoot() . 'backup' . DS . $siteID . DS;
    if(!is_writable(dirname($backupPath)))
    {
        return "Please run the command : chmod o=rwx " . dirname($backupPath);
    }
    if(!is_dir($backupPath)) mkdir($backupPath, 0777, true);

    set_time_limit(7200);

    $sqlFile     = $backupPath . $domain . '.sql';
    $attatchFile = $backupPath . $domain . '.file.zip';

    $zdb = $this->app->loadClass('zdb');
    $return = $zdb->dump($sqlFile);
    if($return->result == false) return $return->error;

    $dataPathList = array();
    $uploadFile = $this->app->getDataRoot() . 'upload/' . $siteID;
    $sourceFile = $this->app->getDataRoot() . 'source/' . $siteID;
    $slidesFile = $this->app->getDataRoot() . 'slides/' . $siteID;
    $cssFile    = $this->app->getDataRoot() . 'css/' . $siteID;
    if(file_exists($uploadFile)) $dataPathList[] = $uploadFile;
    if(file_exists($sourceFile)) $dataPathList[] = $sourceFile;
    if(file_exists($slidesFile)) $dataPathList[] = $slidesFile;
    if(file_exists($cssFile))    $dataPathList[] = $cssFile;

    $this->app->loadClass('pclzip', true);
    $zip = new pclzip($attatchFile);
    $zip->create($dataPathList, PCLZIP_OPT_REMOVE_PATH, $this->app->getDataRoot());
    if($zip->error_code != 0) return false;

    return true;
}

/**
 * getSites 
 * 
 * @access public
 * @return void
 */
public function getSites()
{
    $sites = array();
    $databases = $this->dbh->query("SHOW DATABASES")->fetchAll();
    foreach($databases as $row)
    {
        $database = $row->Database;
        if(strpos($database, 'chanzhi_') === 0)
        {
            $tables = $this->dbh->query("SHOW TABLES FROM $database")->fetchAll();
            if(count($tables) == 0) continue;
            $sites[] = str_replace('chanzhi_', '', $database);
        }
    }
    return $sites;
}

public function getConfigContent()
{
    $node = str_replace(':80', '', $_SERVER['HTTP_HOST']);
    $node = strpos($node, 'www.') === 0 ? substr($node, 4) : $node;

    return <<<EOT
<?php
\$config->installed    = true;
\$config->debug        = false;
\$config->requestType  = 'PATH_INFO';
\$config->http_host    = str_replace(':80', '', \$_SERVER['HTTP_HOST']);
\$config->codePrefix   = '{$this->post->codePrefix}_';
\$config->db->host     = '{$this->post->dbHost}';
\$config->db->port     = '{$this->post->dbPort}';
\$config->db->name     = \$config->codePrefix . '{$node}';
\$config->db->user     = '{$this->post->dbUser}';
\$config->db->password = '{$this->post->dbPassword}';
EOT;
}

public function checkConfig()
{
    $return = new stdclass();
    $return->result = 'ok';

    /* Connect db. */
    $this->setDBParam();

    $node = str_replace(':80', '', $_SERVER['HTTP_HOST']);
    $node = strpos($node, 'www.') === 0 ? substr($node, 4) : $node;

    $this->config->codePrefix = $this->post->codePrefix . '_';
    $this->config->db->name   = "{$this->config->codePrefix}{$node}";
    $this->config->db->prefix = "eps_";

    $this->dbh = $this->connectDB();
    if(!is_object($this->dbh))
    {
        $return->result = 'fail';
        $return->error  = $this->lang->install->errorConnectDB . $this->dbh;
        return $return;
    }
    /* If the db don't exists, try create it. */
    if(!$this->dbExists())
    {

        if(!$this->createDB($node))
        {
            $return->result = 'fail';
            $return->error  = $this->lang->install->errorCreateDB;
            return $return;
        }
    }
    elseif($this->post->clearDB == false)
    {
        $return->result = 'fail';
        $return->error  = $this->lang->install->errorDBExists;
        return $return;
    }

    /* Create the tables. */        
    if(!$this->createTable($node) or !$this->executeSaasSQL($node))
    {
        $return->result = 'fail';       
        $return->error  = $this->lang->install->errorCreateTable;
        return $return;
    }
    return $return;
}

public function copyCustomCss($domain)
{
    $commonPath = $this->app->getWwwRoot() . 'data' . DS . 'css' . DS;
    $cssFiles = glob($commonPath . '*.css');
    $cssPath  =  $this->app->getWwwRoot() . 'sites/' . $domain . '/data/css/';
    if(!is_dir($cssPath)) mkdir($cssPath, 0777, true);
    foreach($cssFiles as $file) 
    {
        copy($file, str_replace($commonPath, $cssPath, $file));
    }
    return true;
}

/**
 * Save the pro sites
 *
 * @access public
 * @param  string $siteID
 * @return void  
 */
public function saveProSite($siteID)
{
    global $config;
    include $this->app->getConfigRoot() . 'ext/sitedomain.php';

    $id2Domain = $config->site->id2Domain;
    $domain    = $id2Domain[$siteID];
    $domain    = strpos($domain, 'www.') === 0 ? substr($domain, 4) : $domain;

    $proSitesListFile = dirname($this->app->getBasePath()) . DS . 'config/prolist.txt';

    $proSiteList = file_get_contents($proSitesListFile);
    $proSiteList = explode(',', $proSiteList);
    if(in_array($domain, $proSiteList)) return true;

    return file_put_contents($proSitesListFile, $domain . ',', FILE_APPEND);
}

/**
 * Remove the pro sites.
 *
 * @access public
 * @param  string $siteID
 * @return void  
 */
public function removeProSite($siteID)
{
    $id2Domain = $this->config->site->id2Domain;
    $domain    = $id2Domain[$siteID];
    $domain    = strpos($domain, 'www.') === 0 ? substr($domain, 4) : $domain;

    $proSitesListFile = dirname($this->app->getBasePath()) . DS . 'config/prolist.txt';

    $proSitesList = file_get_contents($proSitesListFile);
    $proSitesList = str_replace($domain . ',', '', $proSitesList);
    return file_put_contents($proSitesListFile, $proSitesList);
}

/**
 * Save theme code to config table when one site was created.
 * this theme will be installed when the site visited first time.
 * 
 * @param  int    $siteID 
 * @param  code    $theme 
 * @access public
 * @return void
*/
public function setTheme2Import($siteID, $theme)
{
    /* Set tables. */
    $database    = $this->getNameOfDB($siteID);
    $configTable = $database . '.' . $this->config->db->prefix . 'config';

    $item = new stdclass();
    $item->owner   = 'system';
    $item->module  = 'common';
    $item->section = 'global';
    $item->key     = 'theme2Import';
    $item->value   = $theme;
    $item->lang    = 'all';
    $this->dao->replace($configTable)->data($item)->exec();
}

public function importTheme()
{
    $theme = $this->config->global->theme2Import;
    $this->loadModel('ui');

    $template = '';
    if(isset($this->lang->ui->themeList->default[$theme])) $template = 'default';
    if(isset($this->lang->ui->themeList->mobile[$theme]))  $template = 'mobile';
    if(!$template) return true;

    $device = $template == 'default' ? 'desktop' : 'mobile';
    $this->ui->initTheme($device, $template, $theme, 'full');
    $this->ui->setTheme($device, $template, $theme);
    $this->dao->update(TABLE_CONFIG)->set('value')->eq('')->where('`key`')->eq('theme2Import')->exec();
    return true;
}

public function clearData($siteID)
{
    $zfile = $this->app->loadClass('zfile');
    $pathes = array();
    $pathes['uploadPath'] = $this->app->getDataRoot() . 'upload' . DS . $siteID;
    $pathes['sourcePath'] = $this->app->getDataRoot() . 'source' . DS . $siteID;
    $pathes['slidesPath'] = $this->app->getDataRoot() . 'slides' . DS . $siteID;
    $pathes['cssPath']    = $this->app->getDataRoot() . 'css' . DS . $siteID;
    $pathes['cachePath']  = $this->app->getTmpRoot() . 'cache' . DS . $siteID;
    foreach($pathes as $path)
    {
        if(is_dir($path)) $zfile->removeDir($path);
    }
    return true;
}

public function saveMyPHP()
{   
	$configRoot    = $this->app->getConfigRoot();
	$configContent = $this->getConfigContent();

	$return = new stdclass();
	$return->myPHP   = $this->app->getConfigRoot() . 'my.php';
	$proMyPHP        = str_replace("/system/", "/prosystem/", $return->myPHP);
	$return->saved   = is_writable($configRoot) && file_put_contents($return->myPHP, $configContent) && is_writable(dirname($proMyPHP)) && file_put_contents($proMyPHP, $configContent);
	$return->content = $configContent;

	$return->myPHP  .= ", " . $proMyPHP;
	return $return;
}   

